import sys

def main():
    print("You have invoked the `canvas` package directly. You probably want the `canvas.cli` package instead.")
    return 0

if (__name__ == '__main__'):
    sys.exit(main())
